@@ -5,18 +5,17 @@ __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import itertools
-import math
-import operator
 import os
 import weakref
 from collections import namedtuple
+from collections.abc import Iterable
 from functools import wraps
-from io import BytesIO
-from queue import LifoQueue
 from textwrap import wrap
-from threading import Event, Thread
+from threading import Event
+from typing import NamedTuple
 
 from qt.core import (
+    QAbstractItemModel,
     QAbstractItemView,
     QApplication,
     QBuffer,
@@ -57,18 +56,17 @@ from qt.core import (
     qRed,
 )
 
-from calibre import fit_image, human_readable, prepare_string_for_xml
+from calibre import human_readable, prepare_string_for_xml
 from calibre.constants import DEBUG, config_dir, islinux
 from calibre.ebooks.metadata import fmt_sidx, rating_to_stars
 from calibre.gui2 import clip_border_radius, config, empty_index, gprefs, rating_font, resolve_grid_color
 from calibre.gui2.dnd import path_from_qurl
 from calibre.gui2.gestures import GestureManager
-from calibre.gui2.library.caches import CoverCache, ThumbnailCache
+from calibre.gui2.library.caches import CoverThumbnailCache
 from calibre.gui2.library.models import themed_icon_name
+from calibre.gui2.momentum_scroll import MomentumScrollMixin
 from calibre.gui2.pin_columns import PinContainer
-from calibre.utils import join_with_timeout
 from calibre.utils.config import prefs, tweaks
-from calibre.utils.img import convert_PIL_image_to_pixmap
 
 CM_TO_INCH = 0.393701
 CACHE_FORMAT = 'PPM'
@@ -84,6 +82,110 @@ class EncodeError(ValueError):
     pass
 
 
+def cmp(a: int, b: int) -> int:
+    return a - b
+
+
+def row_min(a, b, cmp) -> int:
+    return a if cmp(a, b) <= 0 else b
+
+
+def row_max(a, b, cmp) -> int:
+    return a if cmp(a, b) >= 0 else b
+
+
+def selection_for_rows(m: QAbstractItemModel, rows: Iterable[int]) -> QItemSelection:
+    # Create a range based selector for each set of contiguous rows
+    # as supplying selectors for each individual row causes very poor
+    # performance if a large number of rows has to be selected.
+    sel = QItemSelection()
+    for k, g in itertools.groupby(enumerate(sorted(rows)), lambda i_x: i_x[0]-i_x[1]):
+        smallest = largest = next(g)[1]
+        for x in g:
+            largest = x[1]
+        sel.merge(QItemSelection(m.index(smallest, 0), m.index(largest, 0)), QItemSelectionModel.SelectionFlag.Select)
+    return sel
+
+
+class ClickStartData(NamedTuple):
+    row: int
+    min_row: int
+    max_row: int
+
+
+def double_click_action(index: QModelIndex) -> None:
+    from calibre.gui2.ui import get_gui
+    gui = get_gui()
+    tval = tweaks['doubleclick_on_library_view']
+    if tval == 'open_viewer':
+        gui.iactions['View'].view_triggered(index)
+    elif tval in {'edit_metadata', 'edit_cell'}:
+        gui.iactions['Edit Metadata'].edit_metadata(False, False)
+    elif tval in {'show_book_details', 'show_locked_book_details'}:
+        gui.iactions['Show Book Details'].show_book_info(locked=tval == 'show_locked_book_details')
+
+
+def handle_selection_drag(
+    self: QAbstractItemView, index: QModelIndex, start_data: ClickStartData | None,
+    row_cmp=cmp, selection_between=QItemSelection
+) -> None:
+    if not index.isValid() or start_data is None:
+        return
+    m = self.model()
+    ci = m.index(start_data.row, 0)
+    if not ci.isValid():
+        return
+    sm = self.selectionModel()
+    flags = QItemSelectionModel.SelectionFlag.Rows
+    sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.NoUpdate)
+    if not sm.hasSelection():
+        sm.select(index, QItemSelectionModel.SelectionFlag.ClearAndSelect | flags)
+        return True
+    cr = ci.row()
+    tgt = index.row()
+    if cr == tgt:
+        sm.select(index, QItemSelectionModel.SelectionFlag.Select | flags)
+        return
+    if cr < tgt:
+        # mouse is moved after the start pos
+        top = row_min(start_data.min_row, cr, row_cmp)
+        bottom = tgt
+    else:
+        top = tgt
+        bottom = row_max(start_data.max_row, cr, row_cmp)
+    sm.select(selection_between(m.index(top, 0), m.index(bottom, 0)), QItemSelectionModel.SelectionFlag.ClearAndSelect | flags)
+
+
+def get_click_start_data(self: QAbstractItemView, index: QModelIndex, row_cmp) -> ClickStartData:
+    min_row = self.model().rowCount(QModelIndex())
+    max_row = -1
+    for idx in self.selectionModel().selectedIndexes():
+        r = idx.row()
+        min_row = row_min(r, min_row, row_cmp)
+        max_row = row_max(r, max_row, row_cmp)
+    return ClickStartData(index.row(), min_row, max_row)
+
+
+def handle_selection_click(self: QAbstractItemView, index: QModelIndex, row_cmp=cmp, selection_between=QItemSelection) -> ClickStartData | None:
+    if not index.isValid():
+        return None
+    sm = self.selectionModel()
+    ci = self.currentIndex()
+    flags = QItemSelectionModel.SelectionFlag.Rows
+    if not ci.isValid():
+        return get_click_start_data(self, index, row_cmp)
+    sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.NoUpdate)
+    if not sm.hasSelection():
+        sm.select(index, QItemSelectionModel.SelectionFlag.ClearAndSelect | flags)
+        return get_click_start_data(self, index, row_cmp)
+    cr = ci.row()
+    tgt = index.row()
+    top = self.model().index(row_min(cr, tgt, row_cmp), 0)
+    bottom = self.model().index(row_max(cr, tgt, row_cmp), 0)
+    sm.select(selection_between(top, bottom), QItemSelectionModel.SelectionFlag.Select | flags)
+    return get_click_start_data(self, index, row_cmp)
+
+
 def handle_enter_press(self, ev, special_action=None, has_edit_cell=True):
     if ev.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
         mods = ev.modifiers()
@@ -91,8 +193,11 @@ def handle_enter_press(self, ev, special_action=None, has_edit_cell=True):
             mods & Qt.KeyboardModifier.ControlModifier or mods & Qt.KeyboardModifier.AltModifier or
             mods & Qt.KeyboardModifier.ShiftModifier or mods & Qt.KeyboardModifier.MetaModifier
         ):
-            return
-        if self.state() != QAbstractItemView.State.EditingState and self.hasFocus() and self.currentIndex().isValid():
+            return False
+        s = QAbstractItemView.State.NoState
+        if callable(getattr(self, 'state', None)):
+            s = self.state()
+        if s != QAbstractItemView.State.EditingState and self.hasFocus() and self.currentIndex().isValid():
             from calibre.gui2.ui import get_gui
             ev.ignore()
             tweak = tweaks['enter_key_behavior']
@@ -116,6 +221,44 @@ def handle_enter_press(self, ev, special_action=None, has_edit_cell=True):
                 gui.iactions['View'].view_triggered(self.currentIndex())
             gui.enter_key_pressed_in_book_list.emit(self)
             return True
+    return False
+
+
+def render_emblem(book_id, rule, rule_index, cache, mi, db, formatter, template_cache, column_name='cover_grid'):
+    ans = cache[book_id].get(rule, False)
+    if ans is not False:
+        return ans, mi
+    ans = None
+    if mi is None:
+        mi = db.get_proxy_metadata(book_id)
+    ans = formatter.safe_format(rule, mi, '', mi, column_name=f'{column_name}{rule_index}', template_cache=template_cache) or None
+    cache[book_id][rule] = ans
+    return ans, mi
+
+
+def cached_emblem(sz: int, cache: dict[str, QPixmap | QIcon], name: str, raw_icon=None):
+    ans = cache.get(name, False)
+    if ans is not False:
+        return ans
+    ans = None
+    if raw_icon is not None:
+        ans = raw_icon
+    elif name == ':ondevice':
+        ans = QIcon.cached_icon('ok.png')
+    elif name:
+        d = themed_icon_name(os.path.join(config_dir, 'cc_icons'), name)
+        if d is not None:
+            ans = QIcon(d)
+        if ans is None:
+            ans = QIcon(os.path.join(config_dir, 'cc_icons', name))
+    if ans is not None and not ans.is_ok():
+        ans = None
+    if ans is not None and sz:
+        ans = ans.pixmap(sz, sz)
+        if ans.isNull():
+            ans = None
+    cache[name] = ans
+    return ans
 
 
 def image_to_data(image):  # {{{
@@ -154,8 +297,8 @@ def event_has_mods(self, event=None):
 def mousePressEvent(self, event):
     ep = event.pos()
     # for performance, check the selection only once we know we need it
-    if event.button() == Qt.MouseButton.LeftButton and not self.event_has_mods() \
-                and self.indexAt(ep) in self.selectionModel().selectedIndexes():
+    if event.button() == Qt.MouseButton.LeftButton and not self.event_has_mods(event) \
+                and self.selectionModel().isSelected(self.indexAt(ep)):
         self.drag_start_pos = ep
     if hasattr(self, 'handle_mouse_press_event'):
         return self.handle_mouse_press_event(event)
@@ -243,7 +386,7 @@ def drag_data(self):
 
 
 def mouseMoveEvent(self, event):
-    if self.event_has_mods():
+    if self.event_has_mods() or not event.buttons() & Qt.MouseButton.LeftButton:
         self.drag_start_pos = None
     if not self.drag_allowed:
         if hasattr(self, 'handle_mouse_move_event'):
@@ -457,12 +600,10 @@ class CoverDelegate(QStyledItemDelegate):
         self.animation = QPropertyAnimation(self, b'animated_size', self)
         self.animation.setEasingCurve(QEasingCurve.Type.OutInCirc)
         self.animation.setDuration(500)
-        self.set_dimensions()
-        self.cover_cache = CoverCache()
-        self.render_queue = LifoQueue()
         self.animating = None
         self.highlight_color = QColor(Qt.GlobalColor.white)
         self.rating_font = QFont(rating_font())
+        self.set_dimensions()
 
     def set_dimensions(self):
         width = self.original_width = gprefs['cover_grid_width']
@@ -502,6 +643,14 @@ class CoverDelegate(QStyledItemDelegate):
         self.animation.setStartValue(1.0)
         self.animation.setKeyValueAt(0.5, 0.5)
         self.animation.setEndValue(1.0)
+        dpr = self.parent().device_pixel_ratio
+        w, h = int(dpr * self.cover_size.width()), int(dpr * self.cover_size.height())
+        if hasattr(self, 'cover_cache'):
+            self.cover_cache.set_thumbnail_size(w, h)
+        else:
+            self.cover_cache = CoverThumbnailCache(
+                max_size=gprefs['cover_grid_disk_cache_size'], thumbnail_size=(w, h), parent=self, version=1
+            )
 
     def calculate_spacing(self):
         spc = self.original_spacing = gprefs['cover_grid_spacing']
@@ -532,39 +681,6 @@ class CoverDelegate(QStyledItemDelegate):
                 traceback.print_exc()
         return '', is_stars
 
-    def render_emblem(self, book_id, rule, rule_index, cache, mi, db, formatter, template_cache):
-        ans = cache[book_id].get(rule, False)
-        if ans is not False:
-            return ans, mi
-        ans = None
-        if mi is None:
-            mi = db.get_proxy_metadata(book_id)
-        ans = formatter.safe_format(rule, mi, '', mi, column_name=f'cover_grid{rule_index}', template_cache=template_cache) or None
-        cache[book_id][rule] = ans
-        return ans, mi
-
-    def cached_emblem(self, cache, name, raw_icon=None):
-        ans = cache.get(name, False)
-        if ans is not False:
-            return ans
-        sz = self.emblem_size
-        ans = None
-        if raw_icon is not None:
-            ans = raw_icon.pixmap(sz, sz)
-        elif name == ':ondevice':
-            ans = QIcon.ic('ok.png').pixmap(sz, sz)
-        elif name:
-            pmap = None
-            d = themed_icon_name(os.path.join(config_dir, 'cc_icons'), name)
-            if d is not None:
-                pmap = QIcon(d).pixmap(sz, sz)
-            if pmap is None:
-                pmap = QIcon(os.path.join(config_dir, 'cc_icons', name)).pixmap(sz, sz)
-            if not pmap.isNull():
-                ans = pmap
-        cache[name] = ans
-        return ans
-
     def paint(self, painter, option, index):
         with clip_border_radius(painter, option.rect):
             QStyledItemDelegate.paint(self, painter, option, empty_index)  # draw the hover and selection highlights
@@ -582,15 +698,11 @@ class CoverDelegate(QStyledItemDelegate):
                 painter.drawRoundedRect(option.rect, 10, 10, Qt.SizeMode.RelativeSize)
             finally:
                 painter.restore()
+        cover = self.cover_cache.thumbnail_as_pixmap(book_id)
+        if cover is None:
+            return  # dont draw uncached to avoid flicker
         marked = db.data.get_marked(book_id)
         db = db.new_api
-        cdata = self.cover_cache[book_id]
-        if cdata is False:
-            # Don't render anything if we haven't cached the rendered cover.
-            # This reduces subtle flashing as covers are repainted. Note that
-            # cdata is None if there isn't a cover vs an unrendered cover.
-            self.render_queue.put(book_id)
-            return
         device_connected = self.parent().gui.device_connected is not None
         on_device = device_connected and db.field_for('ondevice', book_id)
 
@@ -599,16 +711,16 @@ class CoverDelegate(QStyledItemDelegate):
         if self.emblem_size > 0:
             mi = None
             for i, (kind, column, rule) in enumerate(emblem_rules):
-                icon_name, mi = self.render_emblem(book_id, rule, i, m.cover_grid_emblem_cache, mi, db, m.formatter, m.cover_grid_template_cache)
+                icon_name, mi = render_emblem(book_id, rule, i, m.cover_grid_emblem_cache, mi, db, m.formatter, m.cover_grid_template_cache)
                 if icon_name is not None:
                     for one_icon in filter(None, (i.strip() for i in icon_name.split(':'))):
-                        pixmap = self.cached_emblem(m.cover_grid_bitmap_cache, one_icon)
+                        pixmap = cached_emblem(self.emblem_size, m.cover_grid_bitmap_cache, one_icon)
                         if pixmap is not None:
                             emblems.append(pixmap)
             if marked:
-                emblems.insert(0, self.cached_emblem(m.cover_grid_bitmap_cache, ':marked', m.marked_icon))
+                emblems.insert(0, cached_emblem(self.emblem_size, m.cover_grid_bitmap_cache, ':marked', m.marked_icon))
             if on_device:
-                emblems.insert(0, self.cached_emblem(m.cover_grid_bitmap_cache, ':ondevice'))
+                emblems.insert(0, cached_emblem(self.emblem_size, m.cover_grid_bitmap_cache, ':ondevice'))
 
         painter.save()
         right_adjust = 0
@@ -622,26 +734,23 @@ class CoverDelegate(QStyledItemDelegate):
             if self.title_height != 0:
                 rect.setBottom(rect.bottom() - self.title_height)
                 trect.setTop(trect.bottom() - self.title_height + 5)
-            if cdata is None or cdata is False:
+            if cover.isNull():
                 title = db.field_for('title', book_id, default_value='')
                 authors = ' & '.join(db.field_for('authors', book_id, default_value=()))
                 painter.setRenderHint(QPainter.RenderHint.TextAntialiasing, True)
                 painter.drawText(rect, Qt.AlignmentFlag.AlignCenter|Qt.TextFlag.TextWordWrap, f'{title}\n\n{authors}')
-                if cdata is False:
-                    self.render_queue.put(book_id)
-                # else if None: don't queue the request
                 if self.title_height != 0:
                     self.paint_title(painter, trect, db, book_id)
             else:
                 if self.animating is not None and self.animating.row() == index.row():
-                    cdata = cdata.scaled(cdata.size() * self._animated_size)
-                dpr = cdata.devicePixelRatio()
-                cw, ch = int(cdata.width() / dpr), int(cdata.height() / dpr)
+                    cover = cover.scaled(cover.size() * self._animated_size)
+                dpr = cover.devicePixelRatio()
+                cw, ch = int(cover.width() / dpr), int(cover.height() / dpr)
                 dx = max(0, int((rect.width() - cw)/2.0))
                 dy = max(0, int((rect.height() - ch)/2.0))
                 right_adjust = dx
                 rect.adjust(dx, dy, -dx, -dy)
-                self.paint_cover(painter, rect, cdata)
+                self.paint_cover(painter, rect, cover)
                 if self.title_height != 0:
                     self.paint_title(painter, trect, db, book_id)
             if self.emblem_size > 0:
@@ -774,15 +883,14 @@ CoverTuple = namedtuple('CoverTuple', ['book_id', 'has_cover', 'cache_valid',
 # The View {{{
 
 @setup_dnd_interface
-class GridView(QListView):
+class GridView(MomentumScrollMixin, QListView):
 
-    update_item = pyqtSignal(object, object)
     files_dropped = pyqtSignal(object)
     books_dropped = pyqtSignal(object)
 
     def __init__(self, parent):
         QListView.__init__(self, parent)
-        self.shift_click_start_data = None
+        self.accumulated_scroll_degrees = 0
         self.dbref = lambda: None
         self._ncols = None
         self.gesture_manager = GestureManager(self)
@@ -799,20 +907,12 @@ class GridView(QListView):
         self.delegate = CoverDelegate(self)
         self.delegate.animation.valueChanged.connect(self.animation_value_changed)
         self.delegate.animation.finished.connect(self.animation_done)
+        self.delegate.cover_cache.rendered.connect(self.re_render)
         self.setItemDelegate(self.delegate)
         self.setSpacing(self.delegate.spacing)
         self.set_color()
         QApplication.instance().palette_changed.connect(self.set_color)
         self.ignore_render_requests = Event()
-        dpr = self.device_pixel_ratio
-        # Up the version number if anything changes in how images are stored in
-        # the cache.
-        self.thumbnail_cache = ThumbnailCache(max_size=gprefs['cover_grid_disk_cache_size'],
-            thumbnail_size=(int(dpr * self.delegate.cover_size.width()),
-                            int(dpr * self.delegate.cover_size.height())),
-            version=1)
-        self.render_thread = None
-        self.update_item.connect(self.re_render, type=Qt.ConnectionType.QueuedConnection)
         self.doubleClicked.connect(self.double_clicked)
         self.setCursor(Qt.CursorShape.PointingHandCursor)
         self.gui = parent
@@ -870,13 +970,7 @@ class GridView(QListView):
 
     def double_clicked(self, index):
         self.start_view_animation(index)
-        tval = tweaks['doubleclick_on_library_view']
-        if tval == 'open_viewer':
-            self.gui.iactions['View'].view_triggered(index)
-        elif tval in {'edit_metadata', 'edit_cell'}:
-            self.gui.iactions['Edit Metadata'].edit_metadata(False, False)
-        elif tval == 'show_book_details':
-            self.gui.iactions['Show Book Details'].show_book_info()
+        double_click_action(index)
 
     def animation_value_changed(self, value):
         if self.delegate.animating is not None:
@@ -924,23 +1018,13 @@ class GridView(QListView):
                         'emblem_position'] != self.delegate.orginal_emblem_position):
             self.delegate.set_dimensions()
             self.setSpacing(self.delegate.spacing)
-            if size_changed:
-                self.delegate.cover_cache.clear()
         if gprefs['cover_grid_spacing'] != self.delegate.original_spacing:
             self.delegate.calculate_spacing()
             self.setSpacing(self.delegate.spacing)
         self.set_color()
-        self.set_thumbnail_cache_image_size()
-        cs = gprefs['cover_grid_disk_cache_size']
-        if (cs*(1024**2)) != self.thumbnail_cache.max_size:
-            self.thumbnail_cache.set_size(cs)
+        self.delegate.cover_cache.set_disk_cache_max_size(gprefs['cover_grid_disk_cache_size'])
         self.update_memory_cover_cache_size()
 
-    def set_thumbnail_cache_image_size(self):
-        dpr = self.device_pixel_ratio
-        self.thumbnail_cache.set_thumbnail_size(
-            int(dpr * self.delegate.cover_size.width()), int(dpr*self.delegate.cover_size.height()))
-
     def resizeEvent(self, ev):
         self._ncols = None
         self.resize_timer.start()
@@ -954,196 +1038,12 @@ class GridView(QListView):
         rows, cols = self.width() // sz.width(), self.height() // sz.height()
         num = (rows + 1) * (cols + 1)
         limit = max(100, num * max(2, gprefs['cover_grid_cache_size_multiple']))
-        if limit != self.delegate.cover_cache.limit:
-            self.delegate.cover_cache.set_limit(limit)
+        self.delegate.cover_cache.set_ram_limit(limit)
 
     def shown(self):
         self.update_memory_cover_cache_size()
-        if self.render_thread is None:
-            self.fetch_thread = Thread(target=self.fetch_covers)
-            self.fetch_thread.daemon = True
-            self.fetch_thread.start()
-
-    def fetch_covers(self):
-        q = self.delegate.render_queue
-        while True:
-            book_id = q.get()
-            try:
-                if book_id is None:
-                    return
-                if self.ignore_render_requests.is_set():
-                    continue
-                thumb = None
-                try:
-                    # Fetch the cover from the cache or file system
-                    cover_tuple = self.fetch_cover_from_cache(book_id)
-                    if cover_tuple is not None:
-                        # Render/resize the cover.
-                        thumb = self.make_thumbnail(cover_tuple)
-                except Exception:
-                    import traceback
-                    traceback.print_exc()
-                # Tell the GUI to redisplay the thumbnail with the new image
-                self.update_item.emit(book_id, thumb)
-
-            finally:
-                q.task_done()
-
-    def fetch_cover_from_cache(self, book_id):
-        '''
-        This method fetches the cover from the cache if it exists, otherwise the
-        cover.jpg stored in the library.
-
-        It is called on the cover thread.
-
-        It returns a CoverTuple containing the following cover and cache data:
-
-        book_id: The id of the book for which a cover is wanted.
-        has_cover: True if the book has an associated cover image file.
-        cdata: Cover data. Can be None (no cover data), or a rendered cover image.
-        cache_valid: True if the cache has correct data, False if a cover exists
-                     but isn't in the cache, None if the cache has data but the
-                     cover has been deleted.
-        timestamp: the cover file modtime if the cover came from the file system,
-                   the timestamp in the cache if a valid cover is in the cache,
-                   otherwise None.
-        '''
-        if self.ignore_render_requests.is_set():
-            return None
-        db = self.dbref()
-        if db is None:
-            return None
-        tc = self.thumbnail_cache
-        cdata, timestamp = tc[book_id]  # None, None if not cached.
-        if timestamp is None:
-            # Cover not in cache. Try to read the cover from the library.
-            has_cover, cdata, timestamp = db.new_api.cover_or_cache(book_id, 0, as_what='pil_image')
-            if has_cover:
-                # There is a cover.jpg, already rendered as a pil_image
-                cache_valid = False
-            else:
-                # No cover.jpg
-                cache_valid = None
-        else:
-            # A cover is in the cache. Check whether it is up to date.
-            # Note that if tcdata is not None then it is already a PIL image.
-            has_cover, tcdata, timestamp = db.new_api.cover_or_cache(book_id, timestamp,
-                                                                     as_what='pil_image')
-            if has_cover:
-                if tcdata is None:
-                    from PIL import Image
-                    # The cached cover is up-to-date. Convert the cached bytes
-                    # to a PIL image
-                    cache_valid = True
-                    cdata = Image.open(BytesIO(cdata))
-                else:
-                    # The cached cover is stale
-                    cache_valid = False
-                    cdata = tcdata
-            else:
-                # We found a cached cover for a book without a cover. This can
-                # happen in older version of calibre that can reuse book_ids
-                # between libraries and books in one library have covers where
-                # they don't in another library. This version doesn't have the
-                # problem because the cache UUID is set when the database
-                # changes instead of when the cache thread is created.
-                tc.invalidate((book_id,))
-                cache_valid = None
-                cdata = None
-        if has_cover and cdata is None:
-            raise RuntimeError('No cover data when has_cover is True')
-        return CoverTuple(book_id=book_id, has_cover=has_cover, cache_valid=cache_valid,
-                          cdata=cdata, timestamp=timestamp)
-
-    def make_thumbnail(self, cover_tuple):
-        # Render the cover image data to the thumbnail size and correct format.
-        # Rendering isn't needed if the cover came from the cache and the cache
-        # is valid. Put newly rendered images into the cache. Returns the
-        # thumbnail as a PIL Image. This method is called on the cover thread.
-
-        cdata = cover_tuple.cdata
-        book_id = cover_tuple.book_id
-        tc = self.thumbnail_cache
-        thumb = None
-        if cover_tuple.has_cover:
-            # cdata contains either the resized thumbnail, the full cover.jpg
-            # rendered as a PIL image, or None if cover.jpg isn't valid
-            if not cover_tuple.cache_valid:
-                # The cover isn't in the cache, is stale, or isn't a valid
-                # image. We might have the image from cover.jpg, in which case
-                # make it into a thumbnail.
-                if cdata is not None:
-                    # We have an image from cover.jpg. Scale it by creating a thumbnail
-                    dpr = self.device_pixel_ratio
-                    page_width = int(dpr * self.delegate.cover_size.width())
-                    page_height = int(dpr * self.delegate.cover_size.height())
-                    scaled, nwidth, nheight = fit_image(cdata.width, cdata.height,
-                                                        page_width, page_height)
-                    if scaled:
-                        # The image is the wrong size. Scale it.
-                        if self.ignore_render_requests.is_set():
-                            return None
-                        # The PIL thumbnail operation works in-place, changing
-                        # the source image.
-                        cdata.thumbnail((int(nwidth), int(nheight)))
-                        thumb = cdata
-                    # Put the new thumbnail into the cache.
-                    try:
-                        with BytesIO() as buf:
-                            cdata.save(buf, format=CACHE_FORMAT)
-                            # use getbuffer() instead of getvalue() to avoid a copy
-                            tc.insert(book_id, cover_tuple.timestamp, buf.getbuffer())
-                        thumb = cdata
-                    except Exception:
-                        tc.invalidate((book_id,))
-                        import traceback
-                        traceback.print_exc()
-                else:
-                    # The cover data isn't valid. Remove it from the cache
-                    tc.invalidate((book_id,))
-            else:
-                # Test to see if there is something wrong with the cover data in
-                # the cache. If so, remove it from the cache and queue it to
-                # render again. It isn't clear that this can ever happen. One
-                # possibility is if different versions of calibre are used
-                # interchangeably.
-                def getbbox(img):
-                    try:
-                        return img.getbbox()
-                    except Exception:
-                        return None
-                if getbbox(cdata) is None:
-                    tc.invalidate((book_id,))
-                    self.render_queue.put(book_id)
-                    return None
-                # The data from the cover cache is valid and is already a thumb.
-                thumb = cdata
-        else:
-            # The book doesn't have a cover.
-            if cover_tuple.cache_valid is not None:
-                # Cover was removed, but it exists in cache. Remove it from the cache
-                tc.invalidate((book_id,))
-            thumb = None
-        # Conversion to QPixmap needs RGBA data. Do it here rather than in the
-        # GUI thread. This check doesn't need to be wrapped in
-        #     if thumb is not None:
-        # because None is a first-class object with no attributes.
-        if getattr(thumb, 'mode', None) == 'RGB':
-            thumb = thumb.convert('RGBA')
-        # Return the thumbnail, which is either None or a PIL Image. If not None
-        # the image will be converted to a QPixmap on the GUI thread. Putting
-        # None into the CoverCache ensures re-rendering won't try again.
-        return thumb
 
     def re_render(self, book_id, thumb):
-        # This is called on the GUI thread when a cover thumbnail is not in the
-        # CoverCache. The parameter "thumb" is either None if there is no cover
-        # or a PIL Image of the thumbnail.
-        self.delegate.cover_cache.clear_staging()
-        if thumb is not None:
-            # Convert the image to a QPixmap
-            thumb = convert_PIL_image_to_pixmap(thumb, self.device_pixel_ratio)
-        self.delegate.cover_cache.set(book_id, thumb)
         m = self.model()
         try:
             index = m.db.row(book_id)
@@ -1152,47 +1052,17 @@ class GridView(QListView):
         self.update(m.index(index, 0))
 
     def shutdown(self):
-        self.ignore_render_requests.set()
-        self.delegate.render_queue.put(None)
-        self.thumbnail_cache.shutdown()
+        self.delegate.cover_cache.shutdown()
 
     def set_database(self, newdb, stage=0):
         self.dbref = weakref.ref(newdb)
         if stage == 0:
-            self.ignore_render_requests.set()
-            try:
-                for x in (self.delegate.cover_cache, self.thumbnail_cache):
-                    self.model().db.new_api.remove_cover_cache(x)
-            except AttributeError:
-                pass  # db is None
-            for x in (self.delegate.cover_cache, self.thumbnail_cache):
-                newdb.new_api.add_cover_cache(x)
-            # This must be done here so the UUID in the cache is changed when
-            # libraries are switched.
-            self.thumbnail_cache.set_database(newdb)
-            try:
-                # Use a timeout so that if, for some reason, the render thread
-                # gets stuck, we don't deadlock, future covers won't get
-                # rendered, but this is better than a deadlock
-                join_with_timeout(self.delegate.render_queue)
-            except RuntimeError:
-                print('Cover rendering thread is stuck!')
-            finally:
-                self.ignore_render_requests.clear()
-        else:
-            self.delegate.cover_cache.clear()
+            self.delegate.cover_cache.set_database(newdb)
 
     def select_rows(self, rows):
-        sel = QItemSelection()
+        sel = selection_for_rows(self.model(), rows)
         sm = self.selectionModel()
-        m = self.model()
-        # Create a range based selector for each set of contiguous rows
-        # as supplying selectors for each individual row causes very poor
-        # performance if a large number of rows has to be selected.
-        for k, g in itertools.groupby(enumerate(rows), lambda i_x: i_x[0]-i_x[1]):
-            group = list(map(operator.itemgetter(1), g))
-            sel.merge(QItemSelection(m.index(min(group), 0), m.index(max(group), 0)), QItemSelectionModel.SelectionFlag.Select)
-        sm.select(sel, QItemSelectionModel.SelectionFlag.ClearAndSelect)
+        sm.select(sel, QItemSelectionModel.SelectionFlag.ClearAndSelect | QItemSelectionModel.SelectionFlag.Rows)
 
     def selectAll(self):
         # We re-implement this to ensure that only indexes from column 0 are
@@ -1233,31 +1103,7 @@ class GridView(QListView):
         # handle shift drag to extend selections
         if not QApplication.keyboardModifiers() & Qt.KeyboardModifier.ShiftModifier:
             return super().mouseMoveEvent(ev)
-        index = self.indexAt(ev.pos())
-        if not index.isValid() or not self.shift_click_start_data:
-            return
-        m = self.model()
-        ci = m.index(self.shift_click_start_data[0], 0)
-        if not ci.isValid():
-            return
-        sm = self.selectionModel()
-        sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.NoUpdate)
-        if not sm.hasSelection():
-            sm.select(index, QItemSelectionModel.SelectionFlag.ClearAndSelect)
-            return True
-        cr = ci.row()
-        tgt = index.row()
-        if cr == tgt:
-            sm.select(index, QItemSelectionModel.SelectionFlag.Select)
-            return
-        if cr < tgt:
-            # mouse is moved after the start pos
-            top = min(self.shift_click_start_data[1], cr)
-            bottom = tgt
-        else:
-            top = tgt
-            bottom = max(self.shift_click_start_data[2], cr)
-        sm.select(QItemSelection(m.index(top, 0), m.index(bottom, 0)), QItemSelectionModel.SelectionFlag.ClearAndSelect)
+        handle_selection_drag(self, self.indexAt(ev.pos()), getattr(self, 'shift_click_start_data', None))
 
     def handle_mouse_press_event(self, ev):
         # Shift-Click in QListView is broken. It selects extra items in
@@ -1267,34 +1113,7 @@ class GridView(QListView):
         # middle item.
         if not QApplication.keyboardModifiers() & Qt.KeyboardModifier.ShiftModifier:
             return super().mousePressEvent(ev)
-        self.shift_click_start_data = None
-        index = self.indexAt(ev.pos())
-        if not index.isValid():
-            return
-        sm = self.selectionModel()
-        ci = self.currentIndex()
-        try:
-            if not ci.isValid():
-                return
-            sm.setCurrentIndex(index, QItemSelectionModel.SelectionFlag.NoUpdate)
-            if not sm.hasSelection():
-                sm.select(index, QItemSelectionModel.SelectionFlag.ClearAndSelect)
-                return
-            cr = ci.row()
-            tgt = index.row()
-            top = self.model().index(min(cr, tgt), 0)
-            bottom = self.model().index(max(cr, tgt), 0)
-            sm.select(QItemSelection(top, bottom), QItemSelectionModel.SelectionFlag.Select)
-        finally:
-            min_row = self.model().rowCount(QModelIndex())
-            max_row = -1
-            for idx in sm.selectedIndexes():
-                r = idx.row()
-                if r < min_row:
-                    min_row = r
-                elif r > max_row:
-                    max_row = r
-            self.shift_click_start_data = index.row(), min_row, max_row
+        self.shift_click_start_data = handle_selection_click(self, self.indexAt(ev.pos()))
 
     def indices_for_merge(self, resolved=True):
         return self.selectionModel().selectedIndexes()
@@ -1394,30 +1213,10 @@ class GridView(QListView):
             return QItemSelectionModel.SelectionFlag.ClearAndSelect | QItemSelectionModel.SelectionFlag.Rows
         return super().selectionCommand(index, event)
 
-    def wheelEvent(self, ev):
-        if ev.phase() not in (Qt.ScrollPhase.ScrollUpdate, Qt.ScrollPhase.NoScrollPhase, Qt.ScrollPhase.ScrollMomentum):
-            return
-        number_of_pixels = ev.pixelDelta()
-        number_of_degrees = ev.angleDelta() / 8.0
-        b = self.verticalScrollBar()
-        if number_of_pixels.isNull() or islinux:
-            # pixelDelta() is broken on linux with wheel mice
-            dy = number_of_degrees.y() / 15.0
-            # Scroll by approximately half a row
-            dy = math.ceil((dy) * b.singleStep() / 2.0)
-        else:
-            dy = number_of_pixels.y()
-        if abs(dy) > 0:
-            b.setValue(b.value() - dy)
-
     def paintEvent(self, ev):
         dpr = self.device_pixel_ratio
         page_width = int(dpr * self.delegate.cover_size.width())
         page_height = int(dpr * self.delegate.cover_size.height())
-        size_changed = self.thumbnail_cache.set_thumbnail_size(page_width, page_height)
-        if size_changed:
-            self.delegate.cover_cache.clear()
-
+        self.delegate.cover_cache.set_thumbnail_size(page_width, page_height)
         return super().paintEvent(ev)
-
 # }}}
