@@ -10,13 +10,12 @@ __docformat__ = 'restructuredtext en'
 import numbers
 import re
 import string
+import sys
 import traceback
 from collections import OrderedDict
-from functools import partial
+from functools import lru_cache, partial
 from math import modf
-from sys import exc_info
 
-from calibre import prints
 from calibre.constants import DEBUG
 from calibre.ebooks.metadata.book.base import field_metadata
 from calibre.utils.config import tweaks
@@ -25,6 +24,24 @@ from calibre.utils.icu import strcmp
 from calibre.utils.localization import _
 from polyglot.builtins import error_message
 
+TEMPLATE_ERROR = _('TEMPLATE ERROR')
+
+
+def default_template_error_reporter(e: Exception, fmt, kwargs, book, column_name: str | None):
+    if DEBUG:
+        if tweaks.get('show_stack_traces_in_formatter', False):
+            traceback.print_exc()
+        if column_name:
+            print(f'Error evaluating column {column_name}:', e, file=sys.stderr)
+
+
+template_error_reporter = default_template_error_reporter
+
+
+def set_template_error_reporter(callback=lambda *a: None):
+    global template_error_reporter
+    template_error_reporter = callback
+
 
 class Node:
     NODE_RVALUE = 1
@@ -1689,6 +1706,40 @@ class _Interpreter:
                        prog.line_number)
 
 
+@lru_cache(maxsize=2)
+def args_scanner() -> re.Scanner:
+    return re.Scanner([
+        (r',', lambda x,t: ''),
+        (r'.*?(?:(?<!\\),)', lambda x,t: t[:-1]),
+        (r'.*?\)', lambda x,t: t[:-1]),
+    ])
+
+
+@lru_cache(maxsize=2)
+def cached_lex_scanner() -> re.Scanner:
+    return re.Scanner([
+        (r'(?:==#|!=#|<=#|<#|>=#|>#)', lambda x,t: (_Parser.LEX_NUMERIC_INFIX, t)),
+        (r'(?:==|!=|<=|<|>=|>)',       lambda x,t: (_Parser.LEX_STRING_INFIX, t)),
+        (r'(?:if|then|else|elif|fi)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),
+        (r'(?:for|in|rof|separator)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),
+        (r'(?:separator|limit)\b',     lambda x,t: (_Parser.LEX_KEYWORD, t)),
+        (r'(?:def|fed|continue)\b',    lambda x,t: (_Parser.LEX_KEYWORD, t)),
+        (r'(?:return|inlist|break)\b', lambda x,t: (_Parser.LEX_KEYWORD, t)),
+        (r'(?:inlist_field)\b',        lambda x,t: (_Parser.LEX_KEYWORD, t)),
+        (r'(?:with|htiw)\b',           lambda x,t: (_Parser.LEX_KEYWORD, t)),
+        (r'(?:\|\||&&|!|{|})',         lambda x,t: (_Parser.LEX_OP, t)),
+        (r'[(),=;:\+\-*/&]',           lambda x,t: (_Parser.LEX_OP, t)),
+        (r'-?[\d\.]+',                 lambda x,t: (_Parser.LEX_CONST, t)),
+        (r'\$\$?#?\w+',                lambda x,t: (_Parser.LEX_ID, t)),
+        (r'\$',                        lambda x,t: (_Parser.LEX_ID, t)),
+        (r'\w+',                       lambda x,t: (_Parser.LEX_ID, t)),
+        (r'".*?(?:(?<!\\)")',          lambda x,t: (_Parser.LEX_CONST, t[1:-1])),
+        (r'\'.*?(?:(?<!\\)\')',        lambda x,t: (_Parser.LEX_CONST, t[1:-1])),
+        (r'\n[ \t]*#.*?(?:(?=\n)|$)',  lambda x,t: _Parser.LEX_NEWLINE),
+        (r'\s',                        lambda x,t: _Parser.LEX_NEWLINE if t == '\n' else None),
+    ], flags=re.DOTALL)
+
+
 class TemplateFormatter(string.Formatter):
     '''
     Provides a format function that substitutes '' for any missing value
@@ -1755,35 +1806,15 @@ class TemplateFormatter(string.Formatter):
     compress_spaces = re.compile(r'\s+')
     backslash_comma_to_comma = re.compile(r'\\,')
 
-    arg_parser = re.Scanner([
-                (r',', lambda x,t: ''),
-                (r'.*?((?<!\\),)', lambda x,t: t[:-1]),
-                (r'.*?\)', lambda x,t: t[:-1]),
-        ])
+    @property
+    def arg_parser(self):
+        return args_scanner()
 
     # ################# Template language lexical analyzer ######################
 
-    lex_scanner = re.Scanner([
-            (r'(==#|!=#|<=#|<#|>=#|>#)', lambda x,t: (_Parser.LEX_NUMERIC_INFIX, t)),
-            (r'(==|!=|<=|<|>=|>)',       lambda x,t: (_Parser.LEX_STRING_INFIX, t)),
-            (r'(if|then|else|elif|fi)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),
-            (r'(for|in|rof|separator)\b',lambda x,t: (_Parser.LEX_KEYWORD, t)),
-            (r'(separator|limit)\b',     lambda x,t: (_Parser.LEX_KEYWORD, t)),
-            (r'(def|fed|continue)\b',    lambda x,t: (_Parser.LEX_KEYWORD, t)),
-            (r'(return|inlist|break)\b', lambda x,t: (_Parser.LEX_KEYWORD, t)),
-            (r'(inlist_field)\b',        lambda x,t: (_Parser.LEX_KEYWORD, t)),
-            (r'(with|htiw)\b',           lambda x,t: (_Parser.LEX_KEYWORD, t)),
-            (r'(\|\||&&|!|{|})',         lambda x,t: (_Parser.LEX_OP, t)),
-            (r'[(),=;:\+\-*/&]',         lambda x,t: (_Parser.LEX_OP, t)),
-            (r'-?[\d\.]+',               lambda x,t: (_Parser.LEX_CONST, t)),
-            (r'\$\$?#?\w+',              lambda x,t: (_Parser.LEX_ID, t)),
-            (r'\$',                      lambda x,t: (_Parser.LEX_ID, t)),
-            (r'\w+',                     lambda x,t: (_Parser.LEX_ID, t)),
-            (r'".*?((?<!\\)")',          lambda x,t: (_Parser.LEX_CONST, t[1:-1])),
-            (r'\'.*?((?<!\\)\')',        lambda x,t: (_Parser.LEX_CONST, t[1:-1])),
-            (r'\n[ \t]*#.*?(?:(?=\n)|$)',lambda x,t: _Parser.LEX_NEWLINE),
-            (r'\s',                      lambda x,t: _Parser.LEX_NEWLINE if t == '\n' else None),
-        ], flags=re.DOTALL)
+    @property
+    def lex_scanner(self):
+        return cached_lex_scanner()
 
     def _eval_program(self, val, prog, column_name, global_vars, break_reporter):
         if column_name is not None and self.template_cache is not None:
@@ -1837,7 +1868,7 @@ class TemplateFormatter(string.Formatter):
         except StopException:
             raise
         except Exception as e:
-            stack = traceback.extract_tb(exc_info()[2])
+            stack = traceback.extract_tb(sys.exc_info()[2])
             ss = stack[-1]
             if getattr(e, 'is_internal', False):
                 # Exception raised by FormatterFuncsCaller
@@ -2080,11 +2111,7 @@ class TemplateFormatter(string.Formatter):
             except StopException as e:
                 ans = error_message(e)
             except Exception as e:
-                if DEBUG:
-                    if tweaks.get('show_stack_traces_in_formatter', True):
-                        traceback.print_exc()
-                    if column_name:
-                        prints('Error evaluating column named:', column_name)
+                template_error_reporter(e, fmt, kwargs, book, column_name)
                 ans = error_value + ' ' + error_message(e)
             return ans
         finally:
